@@ -8,28 +8,25 @@ import (
 	"strings"
 	"time"
 
-	"k8s.io/apimachinery/pkg/apis/meta/v1/unstructured"
-	"k8s.io/apimachinery/pkg/runtime/schema"
-	"k8s.io/client-go/dynamic"
-
-	"k8s.io/klog"
-
-	"k8s.io/apimachinery/pkg/api/equality"
-
-	"github.com/open-cluster-management/multicloud-operators-foundation/pkg/klusterlet/agent"
-
 	"github.com/go-logr/logr"
+	clusterv1beta1 "github.com/open-cluster-management/multicloud-operators-foundation/pkg/apis/internal.open-cluster-management.io/v1beta1"
+	"github.com/open-cluster-management/multicloud-operators-foundation/pkg/klusterlet/agent"
+	routev1 "github.com/openshift/client-go/route/clientset/versioned"
 	"github.com/prometheus/common/log"
 	corev1 "k8s.io/api/core/v1"
+	"k8s.io/apimachinery/pkg/api/equality"
+	"k8s.io/apimachinery/pkg/api/meta"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
+	"k8s.io/apimachinery/pkg/apis/meta/v1/unstructured"
 	"k8s.io/apimachinery/pkg/runtime"
+	"k8s.io/apimachinery/pkg/runtime/schema"
+	"k8s.io/apimachinery/pkg/util/errors"
+	"k8s.io/client-go/dynamic"
 	"k8s.io/client-go/kubernetes"
 	"k8s.io/client-go/tools/cache"
+	"k8s.io/klog"
 	ctrl "sigs.k8s.io/controller-runtime"
 	"sigs.k8s.io/controller-runtime/pkg/client"
-
-	clusterv1beta1 "github.com/open-cluster-management/multicloud-operators-foundation/pkg/apis/cluster/v1beta1"
-	routev1 "github.com/openshift/client-go/route/clientset/versioned"
 )
 
 // ClusterInfoReconciler reconciles a ManagedClusterInfo object
@@ -60,10 +57,8 @@ func (r *ClusterInfoReconciler) Reconcile(req ctrl.Request) (ctrl.Result, error)
 	}
 
 	// Update cluster info status here.
-	newStatus := clusterv1beta1.ClusterInfoStatus{
-		Conditions: clusterInfo.Status.Conditions,
-	}
-
+	newStatus := clusterv1beta1.ClusterInfoStatus{}
+	var errs []error
 	// Config console url
 	_, _, clusterURL := r.getMasterAddresses()
 	newStatus.ConsoleURL = clusterURL
@@ -72,6 +67,7 @@ func (r *ClusterInfoReconciler) Reconcile(req ctrl.Request) (ctrl.Result, error)
 	agentEndpoint, agentPort, err := r.readAgentConfig()
 	if err != nil {
 		log.Error(err, "Failed to get agent server config")
+		errs = append(errs, fmt.Errorf("failed to get agent server config, error:%v ", err))
 	} else {
 		newStatus.LoggingEndpoint = *agentEndpoint
 		newStatus.LoggingPort = *agentPort
@@ -80,10 +76,11 @@ func (r *ClusterInfoReconciler) Reconcile(req ctrl.Request) (ctrl.Result, error)
 	// Get version
 	newStatus.Version = r.getVersion()
 
-	// Get distribution info
-	newStatus.DistributionInfo, err = r.getDistributionInfo()
+	// Get distribution info and ClusterID
+	newStatus.DistributionInfo, newStatus.ClusterID, err = r.getDistributionInfoAndClusterID()
 	if err != nil {
-		log.Error(err, "Failed to get distribution info")
+		log.Error(err, "Failed to get distribution info and clusterID")
+		errs = append(errs, fmt.Errorf("failed to get distribution info and clusterID, error:%v ", err))
 	}
 
 	// Get Vendor
@@ -95,11 +92,44 @@ func (r *ClusterInfoReconciler) Reconcile(req ctrl.Request) (ctrl.Result, error)
 	nodeList, err := r.getNodeList()
 	if err != nil {
 		log.Error(err, "Failed to get nodes status")
+		errs = append(errs, fmt.Errorf("failed to get nodes status, error:%v ", err))
 	} else {
 		newStatus.NodeList = nodeList
 	}
 
-	if !equality.Semantic.DeepEqual(newStatus, clusterInfo.Status) {
+	newSyncedCondition := metav1.Condition{
+		Type:    clusterv1beta1.ManagedClusterInfoSynced,
+		Status:  metav1.ConditionTrue,
+		Reason:  clusterv1beta1.ReasonManagedClusterInfoSynced,
+		Message: "Managed cluster info is synced",
+	}
+	if len(errs) > 0 {
+		newSyncedCondition.Status = metav1.ConditionFalse
+		newSyncedCondition.Reason = clusterv1beta1.ReasonManagedClusterInfoSyncedFailed
+		applyErrors := errors.NewAggregate(errs)
+		newSyncedCondition.Message = applyErrors.Error()
+	}
+
+	needUpdate := false
+	oldStatus := clusterInfo.Status.DeepCopy()
+	oldSyncedCondition := meta.FindStatusCondition(oldStatus.Conditions, clusterv1beta1.ManagedClusterInfoSynced)
+	if oldSyncedCondition != nil {
+		oldSyncedCondition.LastTransitionTime = metav1.Time{}
+		if !equality.Semantic.DeepEqual(newSyncedCondition, *oldSyncedCondition) {
+			needUpdate = true
+		}
+	} else {
+		needUpdate = true
+	}
+
+	oldStatus.Conditions = []metav1.Condition{}
+	if !equality.Semantic.DeepEqual(newStatus, *oldStatus) {
+		needUpdate = true
+	}
+
+	if needUpdate {
+		newStatus.Conditions = clusterInfo.Status.Conditions
+		meta.SetStatusCondition(&newStatus.Conditions, newSyncedCondition)
 		clusterInfo.Status = newStatus
 		err = r.Client.Status().Update(ctx, clusterInfo)
 		if err != nil {
@@ -121,7 +151,7 @@ func (r *ClusterInfoReconciler) getMasterAddresses() ([]corev1.EndpointAddress,
 	}
 
 	if r.MasterAddresses == "" {
-		kubeEndpoints, serviceErr := r.KubeClient.CoreV1().Endpoints("default").Get("kubernetes", metav1.GetOptions{})
+		kubeEndpoints, serviceErr := r.KubeClient.CoreV1().Endpoints("default").Get(context.TODO(), "kubernetes", metav1.GetOptions{})
 		if serviceErr == nil && len(kubeEndpoints.Subsets) > 0 {
 			masterAddresses = kubeEndpoints.Subsets[0].Addresses
 			masterPorts = kubeEndpoints.Subsets[0].Ports
@@ -139,7 +169,7 @@ func (r *ClusterInfoReconciler) getMasterAddressesFromConsoleConfig() ([]corev1.
 	masterPorts := []corev1.EndpointPort{}
 	clusterURL := ""
 
-	cfg, err := r.KubeClient.CoreV1().ConfigMaps("openshift-console").Get("console-config", metav1.GetOptions{})
+	cfg, err := r.KubeClient.CoreV1().ConfigMaps("openshift-console").Get(context.TODO(), "console-config", metav1.GetOptions{})
 	if err == nil && cfg.Data != nil {
 		consoleConfigString, ok := cfg.Data["console-config.yaml"]
 		if ok {
@@ -215,7 +245,7 @@ func (r *ClusterInfoReconciler) setEndpointAddressFromIngress(endpoint *corev1.E
 		log.Error(err, "Failed do parse ingress resource:")
 		return err
 	}
-	klIngress, err := r.KubeClient.ExtensionsV1beta1().Ingresses(klNamespace).Get(klName, metav1.GetOptions{})
+	klIngress, err := r.KubeClient.ExtensionsV1beta1().Ingresses(klNamespace).Get(context.TODO(), klName, metav1.GetOptions{})
 	if err != nil {
 		log.Error(err, "Failed do get ingress resource: %v")
 		return err
@@ -237,7 +267,7 @@ func (r *ClusterInfoReconciler) setEndpointAddressFromService(endpoint *corev1.E
 		return err
 	}
 
-	klSvc, err := r.KubeClient.CoreV1().Services(klNamespace).Get(klName, metav1.GetOptions{})
+	klSvc, err := r.KubeClient.CoreV1().Services(klNamespace).Get(context.TODO(), klName, metav1.GetOptions{})
 	if err != nil {
 		return err
 	}
@@ -268,7 +298,7 @@ func (r *ClusterInfoReconciler) setEndpointAddressFromRoute(endpoint *corev1.End
 		return err
 	}
 
-	route, err := r.RouteV1Client.RouteV1().Routes(klNamespace).Get(klName, metav1.GetOptions{})
+	route, err := r.RouteV1Client.RouteV1().Routes(klNamespace).Get(context.TODO(), klName, metav1.GetOptions{})
 
 	if err != nil {
 		log.Error(err, "Failed to get the route")
@@ -314,7 +344,7 @@ func (r *ClusterInfoReconciler) getVendor(gitVersion string, isOpenShift bool) (
 	}
 
 	cloudVendor = r.getCloudVendor()
-	if cloudVendor == clusterv1beta1.CloudVendorAzure {
+	if cloudVendor == clusterv1beta1.CloudVendorAzure && kubeVendor == clusterv1beta1.KubeVendorOther {
 		kubeVendor = clusterv1beta1.KubeVendorAKS
 	}
 
@@ -322,7 +352,7 @@ func (r *ClusterInfoReconciler) getVendor(gitVersion string, isOpenShift bool) (
 }
 
 func (r *ClusterInfoReconciler) getCloudVendor() clusterv1beta1.CloudVendorType {
-	nodes, err := r.KubeClient.CoreV1().Nodes().List(metav1.ListOptions{})
+	nodes, err := r.KubeClient.CoreV1().Nodes().List(context.TODO(), metav1.ListOptions{})
 	if err != nil {
 		klog.Errorf("failed to get nodes list %v", err)
 		return clusterv1beta1.CloudVendorOther
@@ -365,7 +395,7 @@ const (
 
 func (r *ClusterInfoReconciler) getNodeList() ([]clusterv1beta1.NodeStatus, error) {
 	var nodeList []clusterv1beta1.NodeStatus
-	nodes, err := r.KubeClient.CoreV1().Nodes().List(metav1.ListOptions{})
+	nodes, err := r.KubeClient.CoreV1().Nodes().List(context.TODO(), metav1.ListOptions{})
 	if err != nil {
 		return nil, err
 	}
@@ -415,37 +445,41 @@ func (r *ClusterInfoReconciler) getNodeList() ([]clusterv1beta1.NodeStatus, erro
 	return nodeList, nil
 }
 
+func (r *ClusterInfoReconciler) isOpenshift() bool {
+	serverGroups, err := r.KubeClient.Discovery().ServerGroups()
+	if err != nil {
+		klog.Errorf("failed to get server group %v", err)
+		return false
+	}
+	for _, apiGroup := range serverGroups.Groups {
+		if apiGroup.Name == "project.openshift.io" {
+			return true
+		}
+	}
+	return false
+}
+
 var ocpVersionGVR = schema.GroupVersionResource{
 	Group:    "config.openshift.io",
 	Version:  "v1",
 	Resource: "clusterversions",
 }
 
-func (r *ClusterInfoReconciler) getDistributionInfo() (clusterv1beta1.DistributionInfo, error) {
-	var isOpenShift = false
-	distributionInfo := clusterv1beta1.DistributionInfo{}
-	serverGroups, err := r.KubeClient.Discovery().ServerGroups()
-	if err != nil {
-		return distributionInfo, err
-	}
-	for _, apiGroup := range serverGroups.Groups {
-		if apiGroup.Name == "project.openshift.io" {
-			isOpenShift = true
-			break
-		}
-	}
-	if !isOpenShift {
-		return distributionInfo, nil
-	}
-
-	distributionInfo.Type = clusterv1beta1.DistributionTypeOCP
-	obj, err := r.ManagedClusterDynamicClient.Resource(ocpVersionGVR).Get("version", metav1.GetOptions{})
+func (r *ClusterInfoReconciler) getOCPDistributionInfo() (clusterv1beta1.OCPDistributionInfo, string, error) {
+	ocpDistributionInfo := clusterv1beta1.OCPDistributionInfo{}
+	obj, err := r.ManagedClusterDynamicClient.Resource(ocpVersionGVR).Get(context.TODO(), "version", metav1.GetOptions{})
 	if err != nil {
 		klog.Errorf("failed to get OCP cluster version: %v", err)
-		return distributionInfo, client.IgnoreNotFound(err)
+		return ocpDistributionInfo, "", err
 	}
+
+	clusterID, _, err := unstructured.NestedString(obj.Object, "spec", "clusterID")
+	if err != nil {
+		klog.Errorf("failed to get OCP clusterID in spec: %v", err)
+		return ocpDistributionInfo, "", err
+	}
+
 	historyItems, _, err := unstructured.NestedSlice(obj.Object, "status", "history")
-	var version string
 	if err != nil {
 		klog.Errorf("failed to get OCP cluster version in history of status: %v", err)
 	}
@@ -456,7 +490,7 @@ func (r *ClusterInfoReconciler) getDistributionInfo() (clusterv1beta1.Distributi
 			continue
 		}
 		if state == "Completed" {
-			version, _, err = unstructured.NestedString(historyItem.(map[string]interface{}), "version")
+			ocpDistributionInfo.Version, _, err = unstructured.NestedString(historyItem.(map[string]interface{}), "version")
 			if err != nil {
 				klog.Errorf("failed to get OCP cluster version in latest history of status: %v", err)
 			}
@@ -464,12 +498,11 @@ func (r *ClusterInfoReconciler) getDistributionInfo() (clusterv1beta1.Distributi
 		}
 	}
 
-	desiredVersion, _, err := unstructured.NestedString(obj.Object, "status", "desired", "version")
+	ocpDistributionInfo.DesiredVersion, _, err = unstructured.NestedString(obj.Object, "status", "desired", "version")
 	if err != nil {
 		klog.Errorf("failed to get OCP cluster version in latest history of status: %v", err)
 	}
 	availableUpdates, _, err := unstructured.NestedSlice(obj.Object, "status", "availableUpdates")
-	var availableVersions []string
 	if err != nil {
 		klog.Errorf("failed to get OCP cluster version in latest history of status: %v", err)
 	}
@@ -479,10 +512,10 @@ func (r *ClusterInfoReconciler) getDistributionInfo() (clusterv1beta1.Distributi
 			klog.Errorf("failed to get OCP cluster version in latest history of status: %v", err)
 			continue
 		}
-		availableVersions = append(availableVersions, availableVersion)
+		ocpDistributionInfo.AvailableUpdates = append(ocpDistributionInfo.AvailableUpdates, availableVersion)
 	}
 
-	upgradeFailed := false
+	ocpDistributionInfo.UpgradeFailed = false
 	conditions, _, err := unstructured.NestedSlice(obj.Object, "status", "conditions")
 	if err != nil {
 		klog.Errorf("failed to get OCP cluster version in latest history of status: %v", err)
@@ -500,22 +533,30 @@ func (r *ClusterInfoReconciler) getDistributionInfo() (clusterv1beta1.Distributi
 				continue
 			}
 			if conditionstatus == "True" {
-				upgradeFailed = true
+				ocpDistributionInfo.UpgradeFailed = true
 			}
 			break
 		}
 	}
 
-	distributionInfo.OCP = clusterv1beta1.OCPDistributionInfo{
-		Version: version,
-		// status.desired.version
-		DesiredVersion: desiredVersion,
-		// true when the first status.conditions element with type === Failing has status === True
-		UpgradeFailed: upgradeFailed,
-		// the value of the version property from each element in status.availableUpdates
-		AvailableUpdates: availableVersions,
+	return ocpDistributionInfo, clusterID, nil
+}
+
+func (r *ClusterInfoReconciler) getDistributionInfoAndClusterID() (clusterv1beta1.DistributionInfo, string, error) {
+	var err error
+	var clusterID string
+	var distributionInfo = clusterv1beta1.DistributionInfo{
+		Type: clusterv1beta1.DistributionTypeUnknown,
 	}
-	return distributionInfo, nil
+
+	switch {
+	case r.isOpenshift():
+		distributionInfo.Type = clusterv1beta1.DistributionTypeOCP
+		if distributionInfo.OCP, clusterID, err = r.getOCPDistributionInfo(); err != nil {
+			return distributionInfo, clusterID, err
+		}
+	}
+	return distributionInfo, clusterID, nil
 }
 
 func (r *ClusterInfoReconciler) RefreshAgentServer(clusterInfo *clusterv1beta1.ManagedClusterInfo) {
